# cta.py
import csv

# Hack alert
route_cache = { }

def cached_route(routename):
    if routename not in route_cache:
        route_cache[routename] = routename
    return route_cache[routename]

def read_data(filename):
    records = [ ]
    with open(filename) as file:
        rows = csv.reader(file)
        headers = next(rows)    # Skip the file headers
        for row in rows:
            #row[0] = cached_route(row[0])   # Cache trick
            record = make_record(row)    # ????
            records.append(record)
    return records

# Do nothing.    208 MB of memory (orig file. 15.7MB)
def make_record(row):
    return row

# Tuple. 180MB
def make_record(row):
    return (row[0], row[1], row[2], int(row[3]))

# Dicts. 293MB  (Yikes)
def make_record_dict(row):
    return {
        'route': row[0],
        'date': row[1],
        'daytype': row[2],
        'rides': int(row[3]),
        'a': None,
        'b': None,
        }

# Class. 242 MB (Better than dicts!)
# With slots. 173 MB
class Record(object):
    __slots__ = ('route', 'date', 'daytype', 'rides')
    def __init__(self, route, date, daytype, rides):
        self.route = route
        self.date = date
        self.daytype = daytype
        self.rides = rides

# Avoid typing (at keyboard)
# 186 MB.
from collections import namedtuple
Record = namedtuple('Record', ['route', 'date', 'daytype', 'rides'])

# Newer syntax: (Python 3.6+)
from typing import NamedTuple
class Record(NamedTuple):
    route : str
    date : str
    daytype : str
    rides : int
    
def make_record(row):
    return Record(row[0], row[1], row[2], int(row[3]))

import tracemalloc    # Python 3.4+ 
tracemalloc.start()
# Do something
#make_record = make_record_dict   # Use the dict representation
rows = read_data('ctabus.csv')

current, peak = tracemalloc.get_traced_memory()
print("Peak memory use:", peak)

